# Generated by Django 4.2.17 on 2025-05-12 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0002_alter_baykeshopuser_mobile_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='baykeshopuseraddress',
            name='email',
            field=models.EmailField(blank=True, default='', max_length=254, verbose_name='邮箱'),
        ),
    ]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@文件    :menus.py
@说明    :自定义菜单模型
@时间    :2024/12/12 09:37:13
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
"""

from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import Permission

from baykeshop.db import BaseModel


class BaykeMenus(BaseModel):
    """
    自定义菜单
    """

    name = models.CharField(max_length=50, verbose_name=_("菜单名称"))
    icon = models.CharField(
        max_length=50, blank=True, default="", verbose_name=_("菜单图标")
    )
    parent = models.ForeignKey(
        "self",
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        verbose_name=_("父级菜单"),
    )
    permission = models.OneToOneField(
        Permission,
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        verbose_name=_("权限"),
    )
    is_show = models.BooleanField(default=True, verbose_name=_("是否显示"))
    order = models.IntegerField(default=0, verbose_name=_("排序"))

    class Meta:
        verbose_name = _("自定义菜单")
        verbose_name_plural = _("自定义菜单")
        ordering = ["order"]

    def __str__(self):
        return self.name



from django.urls import path

from . import views

app_name = "member"

urlpatterns = [
    # 登录
    path("login/", views.BaykeShopUserLoginView.as_view(), name="login"),
    # 登出
    path("logout/", views.BaykeShopUserLogoutView.as_view(), name="logout"),
    # 注册
    path("register/", views.BaykeShopUserRegisterView.as_view(), name="register"),
    # 个人中心
    path("profile/", views.BaykeShopUserProfileView.as_view(), name="profile"),
    # 修改个人信息
    path(
        "profile/<int:pk>/update/",
        views.BaykeShopUserProfileUpdateView.as_view(),
        name="profile-update",
    ),
    # 个人中心修改密码
    path(
        "password/", 
        views.BaykeShopUserPasswordView.as_view(), 
        name="password"
    ),
    # 地址列表
    path(
        "address/", 
        views.BaykeShopUserAddressListView.as_view(), 
        name="address-list"
    ),
    # 添加地址
    path(
        "address/create/",
        views.BaykeShopUserAddressCreateView.as_view(),
        name="address-create",
    ),
    # 修改地址
    path(
        "address/<int:pk>/update/",
        views.BaykeShopUserAddressUpdateView.as_view(),
        name="address-update",
    ),
    # 删除地址
    path(
        "address/<int:pk>/delete/",
        views.BaykeShopUserAddressDeleteView.as_view(),
        name="address-delete",
    ),
    # 订单列表
    path(
        "orders/", 
        views.BaykeShopOrdersListView.as_view(), 
        name="orders-list"
    ),
    # 订单详情
    path(
        "orders/<slug:order_sn>/",
        views.BaykeShopOrdersDetailView.as_view(),
        name="orders-detail",
    ),
    # 订单相关操作，订单评论
    path(
        "orders/<slug:order_sn>/comment/",
        views.CommentActionView.as_view(),
        name="orders-comment",
    ),
    # 确认收货
    path(
        "orders/<slug:order_sn>/receipt/",
        views.OrderStatusActionView.as_view(),
        name="orders-receipt",
    ),
    ############### 忘记密码相关操作 START ################
    # 忘记密码
    path(
        "password/reset/",
        views.BaykePasswordResetView.as_view(),
        name="password_reset",
    ),
    # 重置密码
    path(
        "password/reset/done/",
        views.BaykePasswordResetDoneView.as_view(),
        name="password_reset_done",
    ),
    path(
        "password/reset/<uidb64>/<token>/",
        views.BaykePasswordResetConfirmView.as_view(),
        name="password_reset_confirm",
    ),
    # 重置密码完成
    path(
        "password/reset/complete/",
        views.BaykePasswordResetCompleteView.as_view(),
        name="password_reset_complete",
    ),
    ################ 忘记密码相关操作 END ################
]



from django.core import management
from django.core.management.base import BaseCommand
from django.contrib.auth.models import Permission

from baykeshop.contrib.system.models import BaykeMenus


class Command(BaseCommand):
    help = "Create a menus"

    def handle(self, *args, **options):
        self.create_menus()

    def create_menus(self):
        menus = [
            {
                "name": "系统",
                "order": 4,
                "children": [
                    {
                        "name": "菜单管理",
                        "permission": "system.view_baykemenus"
                    },
                    {
                        "name": "字典管理",
                        "permission": "system.view_baykedict"
                    },
                    {
                        "name": "站点设置",
                        "permission": "sites.view_site"
                    },
                    {
                        "name": "轮播图管理",
                        "permission": "system.view_baykebanners"
                    }
                ]
            },
            {
                "name": "商品",
                "order": 1,
                "children": [
                    {
                        "name": "商品分类",
                        "permission": "shop.view_baykeshopcategory"
                    },
                    {
                        "name": "商品管理",
                        "permission": "shop.view_baykeshopgoods"
                    },
                    {
                        "name": "商品规格",
                        "permission": "shop.view_baykeshopspec"
                    },
                    {
                        "name": "品牌管理",
                        "permission": "shop.view_baykeshopbrand"
                    },
                ]
            },
            {
                "name": "订单",
                "order": 2,
                "children": [
                    {
                        "name": "订单管理",
                        "permission": "shop.view_baykeshoporders"
                    },
                    {
                        "name": "订单评论",
                        "permission": "shop.view_baykeshoporderscomment"
                    },
                ]
            },
            {
                "name": "用户",
                "order": 3,
                "children": [
                    {
                        "name": "用户管理",
                        "permission": "auth.view_user"
                    },
                    {
                        "name": "用户组",
                        "permission": "auth.view_group"
                    },
                ]
            },
            {
                "name": "内容",
                "order": 5,
                "children": [
                    {
                        "name": "文章分类",
                        "permission": "article.view_baykearticlecategory"
                    },
                    {
                        "name": "文章管理",
                        "permission": "article.view_baykearticlecontent"
                    },
                    {
                        "name": "标签管理",
                        "permission": "article.view_baykearticletags"
                    },
                    {
                        "name": "侧边栏",
                        "permission": "article.view_baykesidebar"
                    },
                ]
            },
        ]

        for menu in menus:
            menu_obj, iscreated = BaykeMenus.objects.update_or_create(
                name=menu["name"],
                defaults={
                    "name": menu["name"],
                    "order": menu["order"]
                }
            )
            status = '添加' if iscreated else '更新'
            self.stdout.write(self.style.SUCCESS(f'{menu_obj.name} { status } 成功'))

            for child in menu["children"]:
                app_label, codename = child["permission"].split(".")
                permission_obj = Permission.objects.filter(
                    codename=codename, 
                    content_type__app_label=app_label
                )
                if not permission_obj.exists(): continue
                child_obj, _iscreated = BaykeMenus.objects.update_or_create(
                    parent=menu_obj, 
                    name=child["name"], 
                    defaults={
                        "name": child["name"],
                        "parent": menu_obj,
                        "permission": permission_obj.first()
                    }
                )
                status = '添加' if _iscreated else '更新'
                self.stdout.write(self.style.SUCCESS(f'{child_obj.name} { status } 成功'))


# Generated by Django 4.2.17 on 2024-12-10 15:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0002_alter_baykeshoporders_is_verify'),
    ]

    operations = [
        migrations.AlterField(
            model_name='baykeshopgoodssku',
            name='line_price',
            field=models.DecimalField(blank=True, decimal_places=2, default=0.0, help_text='商品价格划线价，不参与价位计算筛选, 仅供前端显示使用', max_digits=10, verbose_name='划线价'),
        ),
        migrations.AlterField(
            model_name='baykeshopgoodssku',
            name='price',
            field=models.DecimalField(decimal_places=2, default=0.0, max_digits=10, verbose_name='价格'),
        ),
    ]



from django.db import models
from django.contrib.auth import get_user_model
from django.contrib.sites.models import Site
from django.contrib.sites.managers import CurrentSiteManager
from django.utils.translation import gettext_lazy as _
from .manager import BaseManager

User = get_user_model()


class BaseModel(models.Model):
    """
    基础模型
    """
    site = models.ForeignKey(
        Site,
        on_delete=models.CASCADE,
        verbose_name=_("站点"),
        editable=False,
        blank=True,
        null=True,
    )
    created_time = models.DateTimeField(auto_now_add=True, verbose_name=_("创建时间"))
    updated_time = models.DateTimeField(auto_now=True, verbose_name=_("更新时间"))
    is_delete = models.BooleanField(
        default=False, editable=False, verbose_name=_("是否删除")
    )

    objects = BaseManager()
    current_site = CurrentSiteManager()

    class Meta:
        abstract = True
        ordering = ["-created_time"]

    def save(self, *args, **kwargs):
        self.site = Site.objects.get_current()
        super().save(*args, **kwargs)



from datetime import timedelta
from django.contrib import messages
from django.utils.translation import gettext_lazy as _
from django.urls import reverse
from django.utils import timezone
from rest_framework import serializers

from baykeshop.contrib.shop.models import BaykeShopOrders
from baykeshop.payment.alipay import TradePagePay


class BaykeShopOrdersPaySerializer(serializers.ModelSerializer):
    """处理支付逻辑
    根据选择的支付方式返回对应的支付地址
    """

    pay_url = serializers.SerializerMethodField()

    class Meta:
        model = BaykeShopOrders
        fields = ("pay_type", "pay_url")

    def validate_pay_type(self, value):
        if value == BaykeShopOrders.PayType.WECHATPAY:
            messages.error(self.context["request"], _("暂不支持微信支付"))
            raise serializers.ValidationError(_("暂不支持微信支付"))
        if self.instance.status != BaykeShopOrders.OrderStatus.UNPAID:
            messages.error(self.context["request"], _("订单状态错误"))
            raise serializers.ValidationError(_("该状态下的订单无法支付, 请联系客服"))
        return value

    def update(self, instance, validated_data):
        # 判断订单创建时间是否在一个小时之内,超时则不能再支付了,这是一种简便的处理方式
        # 后续可以考虑使用消息队列来处理超时订单，这里目前仅做了简单的阻止，库存并未回归
        if (timezone.now() - instance.created_time) > timedelta(hours=1):
            instance.status = BaykeShopOrders.OrderStatus.EXPIRED
            instance.save()
            messages.error(self.context["request"], _("订单已超时, 请重新下单"))
            raise serializers.ValidationError(_("订单已过期"))
        
        order_goods = instance.baykeshopordersgoods_set.first()
        # 是否为虚拟商品
        is_virtual = order_goods.sku.goods.is_virtual if order_goods else False
        pay_type = validated_data.get("pay_type")
        if is_virtual and pay_type == BaykeShopOrders.PayType.CASH:
            # print("虚拟商品不能使用货到付款")
            messages.error(self.context["request"], _("虚拟商品不能使用货到付款"))
            raise serializers.ValidationError(_("虚拟商品不能使用货到付款"))
        if pay_type == BaykeShopOrders.PayType.ALIPAY:
            instance.pay_type = BaykeShopOrders.PayType.ALIPAY
        if pay_type == BaykeShopOrders.PayType.CASH:
            instance.pay_type = BaykeShopOrders.PayType.CASH
            instance.pay_sn = instance.order_sn
            instance.is_verify = True
            instance.status = BaykeShopOrders.OrderStatus.VERIFY
        instance.save()
        return instance

    def get_pay_url(self, instance):
        """获取支付链接"""
        if instance.pay_type == BaykeShopOrders.PayType.CASH:
            messages.success(self.context["request"], _("订单已提交，核销中..."))
            return reverse(
                "member:orders-detail", kwargs={"order_sn": instance.order_sn}
            )

        # 支付宝回调地址
        request = self.context["request"]
        callback_url = request.build_absolute_uri(reverse("shop:alipay-callback"))
        trade_page_pay = TradePagePay(
            request, instance=instance, return_url=callback_url, notify_url=callback_url
        )
        alipay_url = trade_page_pay.pay()
        return alipay_url



import re
import json

from django.core import validators
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


def is_bool(value):
    """
    验证值是否为布尔值
    :param value: 值
    :return: 布尔值
    """
    if value.lower() in ["true", "false"]:
        return True
    return False


def is_dict(value):
    """
    验证值是否为字典
    :param value: 值
    :return: 布尔值
    """
    if all(re.match(r"^\s*\w+\s*:\s*.+\s*$", line) for line in value.splitlines()):
        return True
    return False


def is_list(value):
    """
    验证值是否为列表
    :param value: 值
    :return: 布尔值
    """

    if all(re.match(r"^\s*.+\s*$", line) for line in value.splitlines()):
        return True
    return False


def is_json(value):
    """
    验证值是否为JSON对象
    :param value: 值
    :return: 布尔值
    """
    try:
        json.loads(value)
        return True
    except json.JSONDecodeError:
        return False


def validate_dict_value(value):
    # 布尔值验证
    if is_bool(value):
        return True

    # 键值对验证
    if is_dict(value):
        return True

    # 列表验证
    if is_list(value):
        return True

    # JSON 对象验证
    if is_json(value):
        return True

    raise ValidationError(
        _("%(value)s is not valid"),
        params={"value": value},
        code="invalid",
    )



INSTALLED_APPS = [
    'baykeshop.sites.AdminConfig',
    'baykeshop.contrib.shop',
    'baykeshop.contrib.member',
    'baykeshop.contrib.system',
    'baykeshop.contrib.article'
]

# 版本号
__VERSION__ = "1.3.20"



from .member import (
    BaykeShopUserAddressForm,
    BaykeShopUserProfileForm,
    BaykeShopOrdersCommentForm,
)
from .auth import (
    LoginForm, RegisterForm, ChangePasswordForm, 
    BaykePasswordResetForm, BaykePasswordResetConfirmForm
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@文件    :base.py
@说明    :封装支付宝支付基础类
@时间    :2024/12/10 15:35:39
@作者    :幸福关中&轻编程
@版本    :1.3.15
@微信    :baywanyun
"""

import logging
import traceback

from django.conf import settings
from django.contrib import messages

from alipay.aop.api.AlipayClientConfig import AlipayClientConfig
from alipay.aop.api.DefaultAlipayClient import DefaultAlipayClient
from alipay.aop.api.domain.AlipayTradePagePayModel import AlipayTradePagePayModel
from alipay.aop.api.request.AlipayTradePagePayRequest import AlipayTradePagePayRequest

from baykeshop.contrib.system.models import BaykeDictModel
from baykeshop.contrib.shop.models import BaykeShopOrders


logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s %(levelname)s %(message)s",
    filemode="a",
)

logger = logging.getLogger("")


class Alipay:

    def __init__(
        self,
        appid=None,
        app_private_key=None,
        alipay_public_key=None,
        return_url=None,
        notify_url=None,
        debug=None,
        **kwargs,
    ):
        self.app_id = appid
        self.app_private_key = app_private_key
        self.alipay_public_key = alipay_public_key
        self.return_url = return_url
        self.notify_url = notify_url
        self.debug = debug or settings.DEBUG
        self.extra_params = kwargs

    @property
    def server_url(self):
        if self.debug:
            return "https://openapi-sandbox.dl.alipaydev.com/gateway.do"
        return "https://openapi.alipay.com/gateway.do"

    def config(self, **kwargs):
        _config = AlipayClientConfig()
        _config.app_id = self.app_id or BaykeDictModel.get_key_value("ALIPAY_APPID")
        _config.app_private_key = self.app_private_key or BaykeDictModel.get_key_value(
            "ALIPAY_PRIVATE_KEY"
        )
        _config.alipay_public_key = (
            self.alipay_public_key or BaykeDictModel.get_key_value("ALIPAY_PUBLIC_KEY")
        )
        _config.server_url = self.server_url
        # 参数覆盖
        extra_params = {**self.extra_params, **kwargs}
        _config.sign_type = extra_params.get("sign_type", "RSA2")
        _config.charset = extra_params.get("charset", "utf-8")
        # 其他参数
        for key, value in extra_params.items():
            setattr(_config, key, value)
        return _config

    def client(self, **kwargs):
        """构造一个支付宝请求客户端"""
        _logger = self.extra_params.get("logger") or logger
        return DefaultAlipayClient(
            alipay_client_config=self.config(**kwargs), logger=_logger
        )


class TradePagePay(Alipay):
    """网页支付"""

    def __init__(self, request, instance: BaykeShopOrders = None, **kwargs):
        super().__init__(**kwargs)
        self.instance = instance
        self.request = request

    def pay_model(self, **kwargs):
        """支付参数
        :param kwargs: 自定义支付参数，kwargs优先级高于instance
        :return: AlipayTradePagePayModel实例对象
        """
        if self.instance is None and not kwargs:
            # traceback.print_exc()
            logger.error(f"订单对象或参数必须传入其中一个")
            messages.error(self.request, f"订单对象或参数必须传入其中一个")
            raise ValueError(f"订单对象或参数必须传入其中一个")

        model = AlipayTradePagePayModel()
        model.product_code = "FAST_INSTANT_TRADE_PAY"

        if kwargs:
            try:
                model.out_trade_no = kwargs["out_trade_no"]
                model.total_amount = kwargs["total_amount"]
                model.subject = kwargs["subject"]
                return model
            except KeyError:
                # traceback.print_exc()
                logger.error(f"参数必须包含out_trade_no, total_amount, subject")
                messages.error(self.request, f"参数必须包含out_trade_no, total_amount, subject")
                raise ValueError(f"参数必须包含out_trade_no, total_amount, subject")

        model.out_trade_no = self.instance.order_sn
        model.total_amount = str(self.instance.pay_price)
        model.subject = self.instance.order_sn
        return model

    def pay_request(self, **kwargs):
        """支付请求
        :param kwargs: 支付参数
        :return: AlipayTradePagePayRequest实例对象
        """
        request = AlipayTradePagePayRequest(biz_model=self.pay_model(**kwargs))
        request.return_url = self.return_url
        request.notify_url = self.notify_url
        return request

    def pay(self, **kwargs):
        """支付
        :param kwargs:
            支付参数,当实例未传入instance时必须传入该参数{out_trade_no, total_amount, subject}
        :return: 支付URL
        """
        client = self.client()
        request = self.pay_request(**kwargs)
        response = client.page_execute(request, http_method="GET")
        return response



from django.contrib.auth import get_user_model
from django.contrib.auth.views import (
    LoginView,
    LogoutView,
    PasswordResetView,
    PasswordResetDoneView,
    PasswordResetConfirmView,
    PasswordResetCompleteView
)
from django.contrib.messages.views import SuccessMessageMixin
from django.contrib import messages
from django.views.generic import FormView
from django.utils.translation import gettext_lazy as _
from django.urls import reverse_lazy

from baykeshop.contrib.system.models import BaykeDictModel
from baykeshop.contrib.member.models import BaykeShopUser
from baykeshop.contrib.member.forms import (
    LoginForm, RegisterForm, BaykePasswordResetForm,
    BaykePasswordResetConfirmForm
)

User = get_user_model()


class BaykeShopUserLoginView(SuccessMessageMixin, LoginView):
    """登录页面"""

    template_name = "baykeshop/member/login.html"
    # redirect_authenticated_user = True
    redirect_field_name = "next"
    next_page = reverse_lazy("shop:list")
    form_class = LoginForm
    extra_context = {
        "title": _("登录"),
    }
    success_message = _("登录成功")


class BaykeShopUserRegisterView(FormView):
    """注册页面"""

    template_name = "baykeshop/member/register.html"
    form_class = RegisterForm
    success_url = reverse_lazy("member:login")
    extra_context = {
        "title": _("注册"),
    }

    def form_valid(self, form):
        """注册成功"""
        new_user = form.save(commit=False)
        new_user.set_password(form.cleaned_data["password1"])
        new_user.email = form.cleaned_data["email"]
        user = form.save()
        BaykeShopUser.objects.create(
            user=user, 
            nickname=form.cleaned_data["username"]
        )
        messages.success(self.request, _("注册成功"))
        return super().form_valid(form)


class BaykeShopUserLogoutView(LogoutView):
    """登出页面"""

    next_page = reverse_lazy("member:login")

    def get_success_url(self):
        messages.success(self.request, _("登出成功"))
        return super().get_success_url()


class BaykePasswordResetView(PasswordResetView):
    """重置密码"""
    email_template_name = "baykeshop/member/password_reset_email.html"
    success_url = reverse_lazy("member:password_reset_done")
    template_name = "baykeshop/member/password_reset.html"
    form_class = BaykePasswordResetForm

    
class BaykePasswordResetDoneView(PasswordResetDoneView):
    """重置密码完成"""
    template_name = "baykeshop/member/password_reset_done.html"


class BaykePasswordResetConfirmView(PasswordResetConfirmView):
    """重置密码确认"""
    success_url = reverse_lazy("member:password_reset_complete")
    template_name = "baykeshop/member/password_reset_confirm.html"
    form_class = BaykePasswordResetConfirmForm


class BaykePasswordResetCompleteView(PasswordResetCompleteView):
    """重置密码完成"""
    template_name = "baykeshop/member/password_reset_complete.html"



from django.contrib import admin
from django.urls import reverse, NoReverseMatch

from baykeshop.forms import ModelForm
from baykeshop.contrib.system.models import BaykeDictModel
from baykeshop.db.analysis import (
    UserAnalysisService,
    OrderAnalysisService,
    VisitAnalysisService,
)
from baykeshop.conf import bayke_settings
from baykeshop.contrib.system.models import BaykeMenus
from baykeshop.contrib.shop.models import BaykeShopOrders
from .echarts import orders_chart, users_chart, user_pie_chart


class TabularInline(admin.TabularInline):
    """Tabular Inline View for"""
    form = ModelForm


class StackedInline(admin.StackedInline):
    """Stacked Inline View for"""
    form = ModelForm


class ModelAdmin(admin.ModelAdmin):
    """自定义ModelAdmin"""
    form = ModelForm


class AdminSite(admin.AdminSite):
    """自定义AdminSite"""

    index_template = "baykeshop/admin/index.html"

    @property
    def site_header(self):
        return BaykeDictModel.get_key_value("SITE_HEADER")

    @property
    def site_title(self):
        return BaykeDictModel.get_key_value("SITE_TITLE")

    @property
    def index_title(self):
        return BaykeDictModel.get_key_value("INDEX_TITLE")

    def index(self, request, extra_context=None):
        extra_context = extra_context or {}
        # 用户分析
        user_analysis = UserAnalysisService(request)
        user_data = user_analysis.get_data("%m-%d")
        # 订单分析
        order_analysis = OrderAnalysisService(model=BaykeShopOrders)
        # 订单销售分析
        order_data = order_analysis.get_data("%m-%d")
        sales_data = order_analysis.get_sales_data("%m-%d")
        # 访问分析
        visit_analysis = VisitAnalysisService()
        visit_data = visit_analysis.get_data("%m-%d")

        extra_context = {
            "order_data": order_data,
            "sales_data": sales_data,
            "user_data": user_data,
            "visit_data": visit_data,
            "orders_chart": orders_chart(),
            "users_chart": users_chart(),
            "user_pie_chart": user_pie_chart(),
        }
        return super().index(request, extra_context)

    def each_context(self, request):
        context = super().each_context(request)
        if bayke_settings.USE_MENU:
            context["available_apps"] = self.get_menus(request)
        return context

    def get_menus(self, request):
        # 获取用户的所有权限菜单，页面默认显示view菜单名称
        perms = request.user.get_all_permissions()
        menus_dict = {}
        for perm in perms:
            app_label, codename = tuple(perm.split('.')) 
            menu_qs = BaykeMenus.objects.filter(
                permission__content_type__app_label=app_label, 
                permission__codename=codename,
                parent__isnull=False,
                is_show=True
            )
            if menu_qs.exists():
                _menu = menu_qs.first()
                # model模型
                model = _menu.permission.content_type.model_class()
                is_registered = self.is_registered(_menu.permission.content_type.model_class())
                # 未在admin中注册则跳出循环
                if not is_registered: continue
                # 获取注册的ModelAdmin
                model_admin = self._registry[model]
                # 获取权限
                perms = model_admin.get_model_perms(request)
                # url参数
                info = (app_label, model._meta.model_name)
                if _menu.parent not in menus_dict:
                    menus_dict[_menu.parent] = []
                    menus_dict[_menu.parent].append(_menu)
                else:
                    menus_dict[_menu.parent].append(_menu)

        menus = []

        for parent, values in menus_dict.items():
            model_dict = {
                "name": parent.name,
                "icon": parent.icon,
                "order": parent.order,
                "models": self.get_models(request, values),
            }
            menus.append(model_dict)
        menus = sorted(menus, key=lambda x: x["order"])
        return menus

    def get_models(self, request, values):
        menus = []
        for menu in values:
            item = {}
            item["name"] = menu.name
            item["icon"] = menu.icon
            item["order"] = menu.order
            item["model"] = menu.permission.content_type.model_class()
            item["perms"] = self._registry[item["model"]].get_model_perms(request)
            item["object_name"] = item["model"]._meta.object_name
            app_label = menu.permission.content_type.app_label
            info = (app_label, item["model"]._meta.model_name)
            if item["perms"].get("change") or item["perms"].get("view"):
                item["view_only"] = not item["perms"].get("change")
                try:
                    item["admin_url"] = reverse(
                        "admin:%s_%s_changelist" % info, current_app=self.name
                    )
                except NoReverseMatch:
                    pass
            
            if item["perms"].get("add"):
                try:
                    item["add_url"] = reverse(
                        "admin:%s_%s_add" % info, current_app=self.name
                    )
                except NoReverseMatch:
                    pass
            menus.append(item)
        menus = sorted(menus, key=lambda x: x["order"])
        return menus


admin_site = AdminSite(name="baykeshop")



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@文件    :views.py
@说明    :上传图片视图
@时间    :2024/12/07 17:41:53
@作者    :幸福关中&轻编程
@版本    :1.0
@微信    :baywanyun
"""
from django.conf import settings
from django.core.files.storage import FileSystemStorage

from rest_framework.generics import GenericAPIView
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated
from rest_framework.authentication import SessionAuthentication

from .serializers import UploadImageSerializer


class CsrfExemptSessionAuthentication(SessionAuthentication):
    """关闭csrf验证"""

    def enforce_csrf(self, request):
        return


class UploadImageView(GenericAPIView):
    serializer_class = UploadImageSerializer
    authentication_classes = (CsrfExemptSessionAuthentication,)
    permission_classes = (IsAuthenticated,)

    def post(self, request, *args, **kwargs):
        """上传图片"""
        serializer = self.get_serializer(
            data=request.data, context={"request": request}
        )
        serializer.is_valid(raise_exception=True)
        image = serializer.validated_data["file"]
        # 上传到指定目录
        storeage = FileSystemStorage(
            location=settings.MEDIA_ROOT / "uploads",
            base_url=settings.MEDIA_URL + "uploads/",
        )
        file_name = storeage.save(image.name, image)
        url = storeage.url(file_name)
        return Response({"location": url})



from django.db import models
from .queryset import BaseQuerySet


class BaseManager(models.Manager):
    def get_queryset(self):
        """ 重写父类方法 """
        return BaseQuerySet(self.model, using=self._db)
    
    def delete(self):
        """ 逻辑删除 """
        return self.get_queryset().delete()
    
    def hard_delete(self):
        """ 物理删除 """
        return self.get_queryset().hard_delete()
    
    def restore(self):
        """ 恢复删除 """
        return self.get_queryset().restore()
    
    def deleted(self):
        """ 已删除 """
        return self.get_queryset().deleted()
    
    def all(self):
        """ 未删除 """
        return self.get_queryset().all()
    
    


# Generated by Django 4.2.17 on 2024-12-13 05:47

import django.contrib.sites.managers
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('sites', '0002_alter_domain_unique'),
        ('system', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeMenus',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_time', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='是否删除')),
                ('name', models.CharField(max_length=50, verbose_name='菜单名称')),
                ('icon', models.CharField(blank=True, default='', max_length=50, verbose_name='菜单图标')),
                ('is_show', models.BooleanField(default=True, verbose_name='是否显示')),
                ('order', models.IntegerField(default=0, verbose_name='排序')),
                ('parent', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='system.baykemenus', verbose_name='父级菜单')),
                ('permission', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='auth.permission', verbose_name='权限')),
                ('site', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='sites.site', verbose_name='站点')),
            ],
            options={
                'verbose_name': '自定义菜单',
                'verbose_name_plural': '自定义菜单',
                'ordering': ['order'],
            },
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('current_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



# Generated by Django 4.2.17 on 2025-05-13 13:03

import baykeshop.db.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0005_baykeshoporders_email'),
    ]

    operations = [
        migrations.AddField(
            model_name='baykeshopgoodssku',
            name='email_message',
            field=baykeshop.db.fields.RichTextField(blank=True, default='', verbose_name='虚拟商品邮件内容'),
        ),
    ]



from .auth import (
    BaykeShopUserLoginView,
    BaykeShopUserLogoutView,
    BaykeShopUserRegisterView,
    BaykePasswordResetView,
    BaykePasswordResetDoneView,
    BaykePasswordResetConfirmView,
    BaykePasswordResetCompleteView,
)
from .profile import (
    BaykeShopUserProfileView,
    BaykeShopUserAddressListView,
    BaykeShopUserPasswordView,
    BaykeShopUserAddressCreateView,
    BaykeShopUserAddressUpdateView,
    BaykeShopUserAddressDeleteView,
    BaykeShopUserProfileUpdateView,
)
from .orders import BaykeShopOrdersListView, BaykeShopOrdersDetailView
from .actions import OrderStatusActionView, CommentActionView



# Generated by Django 4.2.17 on 2025-05-07 13:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0003_alter_baykeshopgoodssku_line_price_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='baykeshopgoods',
            name='is_virtual',
            field=models.BooleanField(default=False, help_text='虚拟商品不需要物流', verbose_name='是否为虚拟商品'),
        ),
    ]



from django.db import models


class BaseQuerySet(models.QuerySet):
    
    def delete(self):
        """ 逻辑删除 """
        return super().update(is_delete=True)
    
    def hard_delete(self):
        """ 物理删除 """
        return super().delete()
    
    def restore(self):
        """ 恢复删除 """
        return super().update(is_delete=False)
    
    def deleted(self):
        """ 已删除 """
        return self.filter(is_delete=True)
    
    def undeleted(self):
        """ 未删除 """
        return self.filter(is_delete=False)

    def all(self):
        return self.undeleted()


import json
from django import forms
from django.utils.translation import gettext_lazy as _
from django.contrib.admin import widgets
from baykeshop.forms import ModelForm
from .models import BaykeShopGoodsSKU, BaykeShopSpec


class MyFilteredSelectMultiple(widgets.FilteredSelectMultiple):
    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        if value is not None:
            if isinstance(value, str):
                try:
                    value = json.loads(value)
                    value = [int(v["id"]) for v in value]
                except (json.JSONDecodeError, ValueError):
                    value = []
            elif isinstance(value, list):
                value = [int(v) for v in value]
            else:
                value = [v.pk for v in value]
        context["widget"]["value"] = value
        return context


class BaykeShopGoodsSKUForm(ModelForm):
    """商品SKU表单"""

    specs = forms.ModelMultipleChoiceField(
        queryset=BaykeShopSpec.get_queryset().filter(parent__isnull=False),
        label=_("商品规格"),
        widget=MyFilteredSelectMultiple(_("商品规格"), False),
        required=False,
        help_text=_("单规格时可以不选； 多规格时必选且同一个规格下只能选一个规格值"),
    )

    class Meta:
        model = BaykeShopGoodsSKU
        fields = "__all__"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # 初始化规格值回显
        if self.instance and self.instance.specs:
            try:
                specs_ids = [spec["id"] for spec in json.loads(self.instance.specs)]
                self.initial["specs"] = BaykeShopSpec.get_queryset().filter(
                    id__in=specs_ids
                )
            except (json.JSONDecodeError, ValueError):
                self.initial["specs"] = []

    def clean_specs(self):
        specs = self.cleaned_data.get("specs")
        if specs.exists():
            values = specs.values("id", "parent__name", "name")
            # 父级名称
            parent_names = [spec["parent__name"] for spec in values]
            if specs.count() > len(set(parent_names)):
                raise forms.ValidationError(_("同一个规格下只能选一个规格值"))

            json_data = json.dumps(list(values), ensure_ascii=False)
            specs = json_data
            return specs
        return json.dumps([], ensure_ascii=False)



import json
from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.utils.html import format_html
from django.utils import timezone
from django.template.loader import render_to_string

# Register your models here.
from baykeshop.sites import admin as bayke_admin
from .forms import BaykeShopGoodsSKUForm
from .models import *


class BaykeShopCategoryInline(bayke_admin.TabularInline):
    model = BaykeShopCategory
    extra = 1


@admin.register(BaykeShopCategory)
class BaykeShopCategoryAdmin(bayke_admin.ModelAdmin):
    list_display = ["name", "parent", "order", "is_floor", "is_nav", "is_show"]
    list_editable = ["order", "is_show", "is_floor", "is_nav"]
    list_filter = [
        "is_show",
    ]
    search_fields = ["name"]
    readonly_fields = ["parent"]
    inlines = [BaykeShopCategoryInline]

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "name",
                    "icon",
                    "order",
                    "is_floor",
                    "is_nav",
                    "is_show",
                )
            },
        ),
    )

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "parent":
            kwargs["queryset"] = BaykeShopCategory.objects.filter(parent__isnull=False)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_inline_instances(self, request, obj=None):
        if obj and obj.parent:
            return []
        return super().get_inline_instances(request, obj)


class BaykeShopGoodsSKUInline(bayke_admin.StackedInline):
    model = BaykeShopGoodsSKU
    extra = 1
    form = BaykeShopGoodsSKUForm
    readonly_fields = ("sales",)


class BaykeShopGoodsImagesInline(bayke_admin.TabularInline):
    model = BaykeShopGoodsImages
    extra = 1


@admin.register(BaykeShopGoods)
class BaykeShopGoodsAdmin(bayke_admin.ModelAdmin):
    list_display = (
        "id",
        "name",
        "image",
        "brand",
        "price",
        "sales",
        "stock",
        "is_recommend",
        "is_virtual",
        "created_time",
    )
    list_display_links = ("id", "name", "image")
    list_editable = ("is_recommend",)
    list_filter = ("category", "brand")
    search_fields = ("name", "category__name", "brand__name")
    inlines = [BaykeShopGoodsSKUInline, BaykeShopGoodsImagesInline]
    fieldsets = (
        (
            _("基本信息"),
            {
                "fields": (
                    "name",
                    "category",
                    "brand"
                )
            },
        ),
        (
            _("商品详情"),
            {
                # 'classes': ('collapse',),
                "fields": (
                    "keywords",
                    "description",
                    "detail",
                    "is_virtual"
                )
            },
        ),
    )
    filter_horizontal = ("category",)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        if db_field.name == "category":
            kwargs["queryset"] = BaykeShopCategory.objects.filter(parent__isnull=False)
        return super().formfield_for_manytomany(db_field, request, **kwargs)

    @admin.display(description="商品价格")
    def price(self, obj):
        if obj.price:
            return round(obj.price, 2)
        return None

    @admin.display(description="商品销量")
    def sales(self, obj):
        return obj.sales

    @admin.display(description="商品库存")
    def stock(self, obj):
        return obj.stock

    @admin.display(description="商品图片")
    def image(self, obj):
        return format_html(
            '<img src="/media/{}" width="64" height="64" />', obj.image_url
        )

    # def save_formset(self, request, form, formset, change):
    #     if formset.model == BaykeShopGoodsSKU:
    #         for _form in formset:
    #             if not _form.cleaned_data:
    #                 print(_form.instance, _form.cleaned_data)
    #                 self.message_user(
    #                     request,
    #                     f"【{_form.instance}】的商品规格未设置，请设置后再保存，至少包含一个SKU！",
    #                     level="ERROR",
    #                 )
    #                 break
    #     if formset.model == BaykeShopGoodsImages:
    #         for _form in formset:
    #             if not _form.cleaned_data:
    #                 self.message_user(
    #                     request,
    #                     f"【{_form.instance}】的商品图片未设置，请设置后再保存，至少包含一张图片！",
    #                     level="ERROR",
    #                 )
    #                 break
    #     return super().save_formset(request, form, formset, change)


@admin.register(BaykeShopBrand)
class BaykeShopBrandAdmin(bayke_admin.ModelAdmin):
    """Admin View for BaykeShopBrand"""

    list_display = ("id", "name", "image", "order", "created_time")
    list_display_links = ("id", "name")
    search_fields = ("name", "description")
    list_editable = ("order",)

    fieldsets = (
        (_("基本信息"), {"fields": ("image", "name", "description", "order")}),
    )


class BaykeShopOrdersGoodsInline(bayke_admin.TabularInline):
    model = BaykeShopOrdersGoods
    extra = 0
    exclude = ("specs", "sku", "detail", "image")
    readonly_fields = ("_image", "name", "price", "quantity", "_specs", "sku_sn")

    @admin.display(description="规格")
    def _specs(self, obj):
        specs = obj.specs
        if isinstance(specs, str):
            specs = json.loads(obj.specs)
        if not specs:
            return "-"
        return ", ".join([f"{spec['parent__name']}:{spec['name']}" for spec in specs])

    @admin.display(description="商品图片")
    def _image(self, obj):
        if not obj.image:
            return "-"
        return format_html('<img src="/media/{}" width="64" height="64" />', obj.image)

    def has_delete_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request, obj=None):
        return False


@admin.register(BaykeShopOrders)
class BaykeShopOrdersAdmin(bayke_admin.ModelAdmin):
    list_display = (
        "id",
        "user",
        "order_sn",
        "order_skus",
        "status",
        "pay_type",
        "pay_price",
        "is_verify",
        "is_comment",
        "created_time",
        "pay_time",
    )
    list_display_links = ("id", "user", "order_sn")
    search_fields = ("id", "user__username", "user__nickname")
    list_filter = ("status", "pay_type", "is_verify", "is_comment")
    readonly_fields = (
        "order_sn",
        "user",
        "pay_type",
        "is_comment",
        "pay_sn",
        "pay_time",
        "is_verify",
        "verify_time",
    )
    inlines = [
        BaykeShopOrdersGoodsInline,
    ]
    actions = ["shipments", "verify"]

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        # 未支付和未发货订单可操作修改
        if obj and obj.status in [0]:
            return super().has_change_permission(request, obj)
        return False

    @admin.display(description="订单商品")
    def order_skus(self, obj):
        queryset = obj.baykeshopordersgoods_set.all()
        return render_to_string(
            "baykeshop/admin/ordersgoods.html", {"queryset": queryset}
        )

    @admin.action(description="所选订单 发货")
    def shipments(self, request, queryset):
        for item in queryset:
            if item.status != BaykeShopOrders.OrderStatus.PAID:
                continue
            item.status = BaykeShopOrders.OrderStatus.SHIPPED
            item.save()
        self.message_user(request, "发货成功")

    @admin.action(description="所选订单 核销")
    def verify(self, request, queryset):
        for item in queryset:
            if item.status != BaykeShopOrders.OrderStatus.VERIFY:
                continue
            item.status = BaykeShopOrders.OrderStatus.SIGNED
            item.is_verify = True
            item.pay_time = timezone.now()
            item.save()
        self.message_user(request, "核销成功")


# 规格值
class BaykeShopSpecInline(bayke_admin.TabularInline):
    model = BaykeShopSpec
    extra = 1
    verbose_name = _("规格值")
    verbose_name_plural = _("规格值")


@admin.register(BaykeShopSpec)
class BaykeShopSpecAdmin(bayke_admin.ModelAdmin):
    """Admin View for BaykeShopSpec"""

    list_display = ("id", "name", "parent", "order", "is_show", "created_time")
    list_display_links = ("id", "name")
    search_fields = ("name",)

    fieldsets = ((_("规格名称"), {"fields": ("name", "order", "is_show")}),)
    readonly_fields = ("parent",)
    inlines = [BaykeShopSpecInline]

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "parent":
            kwargs["queryset"] = BaykeShopSpec.objects.filter(parent__isnull=False)
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_inline_instances(self, request, obj=None):
        if obj and obj.parent:
            return []
        return super().get_inline_instances(request, obj)


@admin.register(BaykeShopOrdersComment)
class BaykeShopOrdersCommentAdmin(bayke_admin.ModelAdmin):
    """Admin View for BaykeShopOrdersComment"""

    list_display = (
        "id",
        "user",
        "order",
        "score",
        "content",
        "reply_user",
        "status",
        "created_time",
    )
    list_display_links = ("id", "user", "order")
    list_editable = ("status",)
    search_fields = ("user__username", "user__nickname")
    list_filter = ("score",)
    readonly_fields = ("user", "order", "content", "reply_user", "score")

    fieldsets = (
        (_("评论信息"), {"fields": ("user", "order", "content", "score")}),
        (_("回复信息"), {"fields": ("reply_user", "reply_content", "status")}),
    )

    def save_model(self, request, obj, form, change):
        obj.reply_user = request.user
        return super().save_model(request, obj, form, change)

    def has_add_permission(self, request, obj=None):
        return False



from django.db import models
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from .base import BaseModel
from .fields import RichTextField

User = get_user_model()


class BaseArticleModel(BaseModel):
    """文章基类模型"""

    title = models.CharField(max_length=50, verbose_name=_("标题"))
    content = RichTextField(verbose_name=_("内容"))
    user = models.ForeignKey(
        User, on_delete=models.CASCADE, verbose_name=_("作者"), blank=True, null=True
    )
    image = models.ImageField(
        upload_to="article", blank=True, null=True, verbose_name=_("图片")
    )
    order = models.IntegerField(default=0, verbose_name=_("排序"))

    class Meta:
        ordering = ["-order", "-created_time"]
        abstract = True



# Generated by Django 4.2.17 on 2025-06-10 12:29

import baykeshop.db.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('shop', '0006_baykeshopgoodssku_email_message'),
    ]

    operations = [
        migrations.AlterField(
            model_name='baykeshopgoodssku',
            name='email_message',
            field=baykeshop.db.fields.RichTextField(blank=True, default='', help_text='如果SPU整体为虚拟商品，则填写虚拟商品邮件内容，付款成功后，会发送邮件给用户', verbose_name='虚拟商品邮件内容'),
        ),
    ]



from django.forms import widgets
from django.utils.translation import gettext_lazy as _


class Input(widgets.Input):

    template_name = "baykeshop/forms/widgets/input.html"

    def __init__(self, attrs=None, icon_position=None, icons_class=None):
        super().__init__(attrs)
        # 图标位置bk-has-icons-left bk-has-icons-right
        self.icon_position = icon_position or ""
        self.icons_class = icons_class or {"left": "", "right": ""}

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context["icon_position"] = self.icon_position
        context["icons_class"] = self.icons_class
        return context

    def build_attrs(self, base_attrs, extra_attrs=None):
        extra_attrs.update({"class": "bk-input"})
        return super().build_attrs(base_attrs, extra_attrs)


class TextInput(Input):
    """文本输入框"""

    input_type = "text"
    template_name = "baykeshop/forms/widgets/text.html"


class PasswordInput(Input):
    """密码输入框"""

    input_type = "password"
    template_name = "baykeshop/forms/widgets/password.html"

    def __init__(
        self, attrs=None, icon_position=None, icons_class=None, render_value=False
    ):
        super().__init__(attrs, icon_position, icons_class)
        self.render_value = render_value

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context["render_value"] = self.render_value
        return context


class Select(widgets.Select):
    """下拉框"""

    template_name = "baykeshop/forms/widgets/select.html"

    def __init__(
        self,
        attrs=None,
        choices=(),
        icon_position=None,
        icons_class=None,
        select_class=None,
    ):
        super().__init__(attrs, choices)
        self.icon_position = icon_position or ""
        # 仅支持左侧配置
        self.icons_class = icons_class or {"left": "", "right": ""}
        # select样式,可以影响颜色，大小，状态
        # bk-is-multiple 支持多选
        # bk-is-rounded 圆角
        # bk-is-link等颜色标签，会影响颜色
        # bk-is-small bk-is-medium bk-is-large 大小
        # bk-is-loading 加载中
        # bk-is-hovered 鼠标移上去
        # bk-is-focused 获取焦点
        # bk-is-active 选中
        self.select_class = select_class or ""

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context["icon_position"] = self.icon_position
        context["icons_class"] = self.icons_class
        return context


class RichTextWidget(widgets.Textarea):
    """富文本"""

    template_name = "baykeshop/forms/widgets/richtext.html"

    class Media:
        js = ("baykeshop/tinymce/tinymce.min.js",)



from django import forms
from django.utils.translation import gettext_lazy as _

from baykeshop.forms.mixins import BaseFormMixins
from baykeshop.forms import widgets
from baykeshop.contrib.shop.models import BaykeShopOrdersComment
from baykeshop.contrib.member.models import BaykeShopUserAddress, BaykeShopUser
 

class BaykeShopUserAddressForm(BaseFormMixins, forms.ModelForm):
    """ 收货地址表单 """
    
    class Meta:
        model = BaykeShopUserAddress
        fields = ['name', 'phone', 'province', 'city', 'district', 'address', 'email', 'is_default']
        widgets = {
            'name': widgets.TextInput(
                attrs={'placeholder': _('请输入收货人')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-account', 'right': ''}
            ),
            'phone': widgets.TextInput(
                attrs={'placeholder': _('请输入手机')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-phone', 'right': ''}
            ),
            'province': widgets.TextInput(
                attrs={'placeholder': _('请输入省')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-map-marker', 'right': ''}
            ),
            'city': widgets.TextInput(
                attrs={'placeholder': _('请输入市')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-map-marker', 'right': ''}
            ),
            'district': widgets.TextInput(
                attrs={'placeholder': _('请输入区')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-map-marker', 'right': ''}
            ),
            'address': widgets.TextInput(
                attrs={'placeholder': _('请输入详细地址')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-map-marker', 'right': ''}
            ),
            'email': widgets.TextInput(
                attrs={'placeholder': _('请输入邮箱')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-email', 'right': ''}
            )
        }


class BaykeShopUserProfileForm(BaseFormMixins, forms.ModelForm):
    """ 个人资料表单 """
    
    email = forms.EmailField(
        label=_("邮箱"),
        widget=widgets.TextInput(
            attrs={'placeholder': _('请输入邮箱'), "autocomplete": "email", 'type': 'email'},
            icon_position='bk-has-icons-left',
            icons_class={'left': 'mdi mdi-email', 'right': ''}
        )
    )

    class Meta:
        model = BaykeShopUser
        fields = ['avatar', 'gender', 'nickname', 'birthday', 'email', 'mobile', 'qq', 'wechat', 'description']
        widgets = {
            'nickname': widgets.TextInput(
                attrs={'placeholder': _('请输入昵称')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-account', 'right': ''},
            ),
            'mobile': widgets.TextInput(
                attrs={'placeholder': _('请输入手机号码')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-phone', 'right': ''}
            ),
            'gender': widgets.Select(
                attrs={'placeholder': _('请选择性别')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-gender-male-female', 'right': ''}
            ),
            'birthday': forms.DateInput(attrs={'type': 'date', 'class': 'bk-input' }, format='%Y-%m-%d'),
            'qq': widgets.TextInput(
                attrs={'placeholder': _('请输入QQ')},
               icon_position='bk-has-icons-left',
               icons_class={'left': 'mdi mdi-qqchat', 'right': ''}
            ),
            'wechat': widgets.TextInput(
                attrs={'placeholder': _('请输入微信')},
                icon_position='bk-has-icons-left',
                icons_class={'left': 'mdi mdi-wechat', 'right': ''}
            ),
            'description': forms.Textarea(
                attrs={'placeholder': _('请输入简介'), 'class': 'bk-textarea bk-has-fixed-size'},
            )
        }


class BaykeShopOrdersCommentForm(BaseFormMixins, forms.ModelForm):
    """订单评论表单"""

    class Meta:
        model = BaykeShopOrdersComment
        fields = ('content', 'score',)
        widgets = {
            'content': forms.Textarea(
                attrs={'placeholder': _('请输入评论内容'), 'class': 'bk-textarea', 'rows': 5, 'cols': 50},
            ),
            'score': widgets.Select(
                attrs={'placeholder': _('请选择评分')},
                icon_position='bk-has-icons-left',
                icons_class={'left':'mdi mdi-star', 'right': ''}
            )
        }


import logging

from django import forms
from django.contrib.auth.forms import (
    UserCreationForm,
    AuthenticationForm,
    PasswordChangeForm,
    PasswordResetForm,
    SetPasswordForm
)
from django.contrib.auth import password_validation, get_user_model
from django.utils.translation import gettext_lazy as _
from django.template import loader
from django.core.mail import get_connection
from django.core.mail import EmailMultiAlternatives
from django.conf import settings

from baykeshop.contrib.system.models import BaykeDictModel
from baykeshop.forms.mixins import BaseFormMixins
from baykeshop.forms import widgets

logger = logging.getLogger("baykeshop.contrib.member")


class LoginForm(BaseFormMixins, AuthenticationForm):
    """登录表单"""

    username = forms.CharField(
        label=_("用户名"),
        widget=widgets.TextInput(
            attrs={"placeholder": _("请输入用户名"), "autofocus": True},
            icon_position="bk-has-icons-left bk-has-icons-right",
            icons_class={"left": "mdi mdi-account", "right": "mdi mdi-check"},
        ),
        required=True,
    )
    password = forms.CharField(
        label=_("密码"),
        widget=widgets.PasswordInput(
            attrs={"placeholder": _("请输入密码")},
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-lock", "right": ""},
        ),
        required=True,
    )


class RegisterForm(BaseFormMixins, UserCreationForm):
    """注册表单"""

    username = forms.CharField(
        label=_("用户名"),
        strip=False,
        widget=widgets.TextInput(
            attrs={"placeholder": _("请输入用户名"), "autofocus": True},
            icon_position="bk-has-icons-left bk-has-icons-right",
            icons_class={"left": "mdi mdi-account", "right": "mdi mdi-check"},
        ),
        max_length=150,
        min_length=6,
        help_text=_("请输入6-16位用户名，只能包含字母、数字、下划线"),
    )
    email = forms.EmailField(
        label=_("邮箱"),
        widget=widgets.TextInput(
            attrs={
                "placeholder": _("请输入邮箱"),
                "autocomplete": "email",
                "type": "email",
            },
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-email", "right": ""},
        ),
        required=True,
        help_text=_("请输入正确的邮箱地址，用于虚拟商品接收邮件"),
    )
    password1 = forms.CharField(
        label=_("Password"),
        widget=widgets.PasswordInput(
            attrs={"placeholder": _("请输入密码"), "autocomplete": "new-password"},
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-lock", "right": ""},
        ),
        strip=False,
        help_text=password_validation.password_validators_help_text_html(),
    )
    password2 = forms.CharField(
        label=_("Password confirmation"),
        widget=widgets.PasswordInput(
            attrs={"placeholder": _("请再次输入密码")},
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-lock", "right": ""},
        ),
        help_text=_("Enter the same password as before, for verification."),
    )

    def clean_email(self):
        email = self.cleaned_data["email"]
        print(email)
        User = get_user_model()
        if User.objects.filter(email=email).exists():
            raise forms.ValidationError(
                _("邮箱已存在"), code="email_exists"
            )
        return email


class ChangePasswordForm(BaseFormMixins, PasswordChangeForm):
    """修改密码表单"""

    old_password = forms.CharField(
        label=_("Old password"),
        strip=False,
        widget=widgets.PasswordInput(
            attrs={
                "placeholder": _("请输入旧密码"),
                "autocomplete": "current-password",
            },
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-lock", "right": ""},
        ),
    )
    new_password1 = forms.CharField(
        label=_("New password"),
        widget=widgets.PasswordInput(
            attrs={"placeholder": _("请输入新密码"), "autocomplete": "new-password"},
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-lock", "right": ""},
        ),
        strip=False,
        help_text=password_validation.password_validators_help_text_html(),
    )
    new_password2 = forms.CharField(
        label=_("New password confirmation"),
        strip=False,
        widget=widgets.PasswordInput(
            attrs={
                "placeholder": _("请再次输入新密码"),
                "autocomplete": "new-password",
            },
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-lock", "right": ""},
        ),
        help_text=_("Enter the same password as before, for verification."),
    )


class BaykePasswordResetForm(BaseFormMixins, PasswordResetForm):
    """重置密码表单"""

    email = forms.EmailField(
        label=_("邮箱"),
        widget=widgets.TextInput(
            attrs={
                "placeholder": _("请输入邮箱"),
                "autocomplete": "email",
                "type": "email",
            },
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-email", "right": ""},
        ),
    )

    def send_mail(
        self,
        subject_template_name,
        email_template_name,
        context,
        from_email,
        to_email,
        html_email_template_name=None,
    ):
        """
        Send a django.core.mail.EmailMultiAlternatives to `to_email`.
        """
        subject = loader.render_to_string(subject_template_name, context)
        # Email subject *must not* contain newlines
        subject = "".join(subject.splitlines())
        body = loader.render_to_string(email_template_name, context)
        # 指定邮件发送服务器
        DEVELOP_EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"
        PRODUCTION_EMAIL_BACKEND = "django.core.mail.backends.smtp.EmailBackend"
        from_email = BaykeDictModel.get_key_value("EMAIL_HOST_USER")
        connection = get_connection(
            fail_silently=False,
            host=BaykeDictModel.get_key_value("EMAIL_HOST"),
            port=int(BaykeDictModel.get_key_value("EMAIL_PORT")),
            username=BaykeDictModel.get_key_value("EMAIL_HOST_USER"),
            password=BaykeDictModel.get_key_value("EMAIL_HOST_PASSWORD"),
            use_ssl=BaykeDictModel.get_key_value("EMAIL_USE_SSL"),
            from_email=from_email,
            backend=PRODUCTION_EMAIL_BACKEND if not settings.DEBUG else DEVELOP_EMAIL_BACKEND,
        )
        email_message = EmailMultiAlternatives(subject, body, from_email, [to_email], connection=connection)
        if html_email_template_name is not None:
            html_email = loader.render_to_string(html_email_template_name, context)
            email_message.attach_alternative(html_email, "text/html")

        try:
            email_message.send()
        except Exception:
            logger.exception(
                "Failed to send password reset email to %s", context["user"].pk
            )


class BaykePasswordResetConfirmForm(BaseFormMixins, SetPasswordForm):
    """重置密码确认表单"""

    new_password1 = forms.CharField(
        label=_("New password"),
        strip=False,
        widget=widgets.PasswordInput(
            attrs={"placeholder": _("请输入新密码"), "autocomplete": "new-password"},
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-lock", "right": ""},
        )
    )
    new_password2 = forms.CharField(
        label=_("New password confirmation"),
        strip=False,
        widget=widgets.PasswordInput(
            attrs={
                "placeholder": _("请再次输入新密码"),
                "autocomplete": "new-password",
            },
            icon_position="bk-has-icons-left",
            icons_class={"left": "mdi mdi-lock", "right": ""},
        )
    )